# coding=utf-8
from oyeahz_base.logger.xlogger import logger
from flask import Request, Response
import json


# 获取query参数
def get_query_param(req: Request, key):
    param = req.args.get(key)
    if param is None:
        raise Exception(f'key: %s error!' % key)
    return param


# 获取query参数，不抛异常返回None
def safe_get_query_param(req: Request, key):
    param = None
    try:
        param = req.args.get(key)
    except Exception as obj:
        logger.error(obj)
    return param


# 获取get_data里的body数据
def get_post_param(req: Request, key):
    post_body = req.get_data()
    body_data = json.loads(post_body)
    return body_data[key]


# 获取get_data里的body数据，不抛异常返回None
def safe_get_post_param(req: Request, key):
    param = None
    try:
        post_body = req.get_data()
        body_data = json.loads(post_body)
        param = body_data[key]
    except Exception as obj:
        logger.error(obj)
    return param


# 把Model转成响应
def make_response(resp: Response) -> str:
    return json.dumps(resp, default=lambda obj: obj.__dict__)



# coding=utf-8

class BaseResponse:
    def __init__(self):
        self.error_code = -1
        self.error_msg = ""
        self.data = {}


class ParamErrorResponse:
    def __init__(self):
        self.error_code = 499
        self.error_msg = "参数异常，请稍候再试"
        self.data = None


class SeverErrorResponse:
    def __init__(self):
        self.error_code = 500
        self.error_msg = "系统开小差了"
        self.data = None


class SessionInvalidResponse:
    def __init__(self):
        self.error_code = 600
        self.error_msg = "会话过期，请重新登录"
        self.data = None



# coding=utf-8
import redis
from redis.client import Redis
from oyeahz_base.utils import oyeahz_utils
from oyeahz_base.logger.xlogger import logger


# 获取redis实例
def get_redis(host: str, port: int) -> Redis:
    return redis.StrictRedis(host=host, port=port, db=0)


# 通过本地配置获取Redis实例
def get_redis_by_local_key(key: str) -> Redis:
    redis = None
    try:
        host = oyeahz_utils.get_local_config(key, 'host')
        port = oyeahz_utils.get_local_config(key, 'port')
        redis = get_redis(host, port)
    except Exception as obj:
        logger.error(obj)
    return redis



# coding=utf-8
import hashlib
import random
import re
import string
import time
import os
import configparser
from oyeahz_base.logger.xlogger import logger


# 判断手机号是否有效
def is_phone_number_valid(phone_num: str) -> bool:
    if phone_num is None:
        return False
    if len(str(phone_num)) != 11:
        flag = False
    else:
        if not str(phone_num).isdigit():
            flag = False
        else:
            phone_rule = re.compile('^0\d{2,3}\d{7,8}$|^1[358]\d{9}$|^147\d{8}')
            res_list = re.findall(phone_rule, str(phone_num))
            if res_list:
                flag = True
            else:
                flag = False
    if flag:
        return True
    else:
        return False


# 判断邮箱是否有效
def is_email_valid(email: str) -> bool:
    if email is None:
        return False
    if re.match("^.+\\@(\\[?)[a-zA-Z0-9\\-\\.]+\\.([a-zA-Z]{2,3}|[0-9]{1,3})(\\]?)$", email) is not None:
        return True
    else:
        return False


# 获取任意长度的随机码
def get_random_code(length : int) -> str:
    source = '123456789'
    code = ''
    for i in range(length):
        num = random.randint(0, len(source) - 1)
        code += source[num]
    return code


# 创建一个随机长度的session
def gen_random_session(length: int) -> str:
    return ''.join(random.sample(string.ascii_letters + string.digits, length))


# 获取字符串MD5
def get_md5(input: str):
    return hashlib.md5(input.encode(encoding='UTF-8')).hexdigest()


# 当前时间，格式：年-月-日 时:分:秒
def get_current_time() -> str:
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime())


# 当前时间戳 毫秒级
def get_local_time_millis() -> int:
    return int(time.time() * 1000)


# 根据时间戳获取当前时间，格式：年-月-日 时:分:秒
def get_time(time_millis: int) -> str:
    return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(time_millis / 1000))


# 根据时间字符串转为时间戳 秒级
def get_timestamp(time_str: str) -> int:
    return int(time.mktime(time.strptime(time_str, '%Y-%m-%d %H:%M:%S')))


# 根据时间戳获取当前日期，格式：年-月-日
def get_date(time_millis: int) -> str:
    return time.strftime("%Y-%m-%d", time.localtime(time_millis / 1000))


# 从字典获取值，不存在的key返回None
def dict_safe_get(input_dict: {}, key: str):
    param = None
    try:
        param = input_dict[key]
    except Exception as obj:
        logger.error(obj)
    return param


# 从数组获取值，不存在的index返回None
def array_safe_get(input_array: [], index: int):
    param = None
    try:
        param = input_array[index]
    except Exception as obj:
        logger.error(obj)
    return param


# 判断字符串是否空
def string_is_empty(input_str: str) -> bool:
    if input_str is None or input_str == '':
        return True
    return False


# 定义文件配置读取的类，oyeahz的本地配置全用这个配置文件
class ReadConfig:

    def __init__(self, filepath=None):
        if filepath:
            config_path = filepath
        else:
            root_dir = os.path.abspath('.')
            config_path = os.path.join(root_dir, "config.ini")
        self.cf = configparser.ConfigParser()
        self.cf.read(config_path)

    def get_config(self, key: str, param: str):
        value = self.cf.get(key, param)
        return value


# 根据本地文件配置的key取param对应的内容
def get_local_config(key: str, param: str):
    read_config = ReadConfig()
    return read_config.get_config(key, param)




# coding=utf-8
import logging
import sys
from os.path import join, dirname
from datetime import datetime


def today():
    return datetime.now().strftime('%Y_%m_%d')


def set_up_logger():
    # 获取logger实例，如果参数为空则返回root logger
    _logger = logging.getLogger("Oyeahz")

    # 指定logger输出格式
    formatter = logging.Formatter('%(asctime)s %(levelname)-8s: %(message)s')

    # 文件日志
    file_handler = logging.FileHandler(join(dirname(dirname(__file__)), f"oyeahz_{today()}.log"), encoding='utf-8')
    file_handler.setFormatter(formatter)  # 可以通过setFormatter指定输出格式

    # 控制台日志
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.formatter = formatter  # 也可以直接给formatter赋值

    # 为logger添加的日志处理器
    _logger.addHandler(file_handler)
    _logger.addHandler(console_handler)

    # 指定日志的最低输出级别，默认为WARN级别
    _logger.setLevel(logging.INFO)

    return _logger


logger = set_up_logger()



# coding=utf-8
import pymysql
import re
from pymysql.connections import Connection
from oyeahz_base.logger.xlogger import logger
from oyeahz_base.utils import oyeahz_utils


# 获取mysql数据库实例
def get_database(host: str, user: str, password: str, database: str) -> Connection:
    db = None
    try:
        db = pymysql.connect(host=host, user=user, password=password, database=database, charset='utf8')
    except Exception as obj:
        logger.error(obj)
    return db


# 通过本地配置获取mysql数据库实例
def get_database_by_local_key(key: str, database: str) -> Connection:
    db = None
    try:
        host = oyeahz_utils.get_local_config(key, 'host')
        user = oyeahz_utils.get_local_config(key, 'user')
        password = oyeahz_utils.get_local_config(key, 'password')
        db = get_database(host, user, password, database)
    except Exception as obj:
        logger.error(obj)
    return db


# 判断数据库中表是否存在
def table_exists(database: Connection, table_name: str) -> bool:
    if database is None:
        return False
    try:
        cursor = database.cursor()
        show_table_sql = "show tables;"
        cursor.execute(show_table_sql)
        tables = [cursor.fetchall()]
        table_list = re.findall('(\'.*?\')', str(tables))
        table_list = [re.sub("'", '', each) for each in table_list]
        if table_name in table_list:
            return True
        else:
            return False
    except Exception as obj:
        logger.error(obj)


def where_and_condition(where_info: dict) -> str:
    sql = ""
    where_condition_list = where_info.keys()
    if len(where_condition_list) > 0:
        sql = " WHERE"
        for column in where_condition_list:
            value = where_info[column]
            if type(value) is int or type(value) is float:
                sql += " " + column + "=" + str(value) + ""
            else:
                sql += " " + column + "=\'" + where_info[column] + "\'"
            sql += " AND"
        sql = sql[:-4]
    return sql


def where_or_condition(where_info: dict) -> str:
    sql = ""
    where_condition_list = where_info.keys()
    if len(where_condition_list) > 0:
        sql = " WHERE"
        for column in where_condition_list:
            value = where_info[column]
            if type(value) is int or type(value) is float:
                sql += " " + column + "=" + str(value) + ""
            else:
                sql += " " + column + "=\'" + where_info[column] + "\'"
            sql += " OR"
        sql = sql[:-4]
    return sql


def or_condition(where_info: dict) -> str:
    sql = ""
    where_condition_list = where_info.keys()
    if len(where_condition_list) > 0:
        sql = ""
        for column in where_condition_list:
            value = where_info[column]
            if type(value) is int or type(value) is float:
                sql += column + "=" + str(value) + ""
            else:
                sql += column + "=\'" + where_info[column] + "\'"
            sql += " OR "
        sql = sql[:-4]
    return sql


def and_condition(where_info: dict) -> str:
    sql = ""
    where_condition_list = where_info.keys()
    if len(where_condition_list) > 0:
        sql = ""
        for column in where_condition_list:
            value = where_info[column]
            if type(value) is int or type(value) is float:
                sql += column + "=" + str(value) + ""
            else:
                sql += column + "=\'" + where_info[column] + "\'"
            sql += " AND "
        sql = sql[:-5]
    return sql


def gen_insert_sql(table_name: str, column_info: dict) -> str:
    column_list = column_info.keys()
    sql = 'INSERT INTO %s (' % table_name
    for column in column_list:
        sql += "'" + column + "',"
    sql = sql[:-1]
    sql += ") VALUES ("
    for column in column_list:
        value = column_info[column]
        if type(value) is int or type(value) is float:
            sql += str(value) + ","
        else:
            sql += "\'" + value + "\',"
    sql = sql[:-1] + ");"
    return sql


def gen_update_sql(table_name: str, update_info: dict, where_info: dict) -> str:
    column_list = update_info.keys()
    sql = "UPDATE %s SET " % table_name
    for column in column_list:
        value = update_info[column]
        if type(value) is int or type(value) is float:
            sql += column + "=" + str(update_info[column]) + ","
        else:
            sql += column + "=\'" + str(update_info[column]) + "\',"
    sql = sql[:-1]
    sql += where_and_condition(where_info)
    sql += ";"
    return sql


# 生成查询sql
def gen_single_select_sql(table_name: str, column_list: list, where_info: dict) -> str:
    sql = "SELECT "
    if len(column_list) > 0:
        sql += ','.join(str(column) for column in column_list)
    else:
        sql += "*"
    sql += " FROM %s " % table_name
    sql += where_and_condition(where_info)
    sql += ";"
    return sql


def gen_delete_sql(table_name, where_info: dict) -> str:
    sql = "DELETE %s " % table_name
    sql += where_and_condition(where_info)
    sql += ";"
    return sql
